/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.oag.context;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import bean.pta.Heap;

/**
 * In BEAN, a context is a sequence of heaps.
 *
 */
public class Context {

	private final List<Heap> context;
	private int hashCode = 0;

	private Context(List<Heap> context) {
		this.context = context;
	}

	public boolean contains(Heap heap) {
		return context.contains(heap);
	}
	
	/**
	 * 
	 * @param i
	 * @return the i-th heap of this context
	 */
	public Heap get(int i) {
		return context.get(i);
	}

	public int depth() {
		return context.size();
	}

	public boolean isEmpty() {
		return context.isEmpty();
	}
	
	public Iterator<Heap> iterator() {
		return context.iterator();
	}

	@Override
	public int hashCode() {
		if (hashCode == 0) {
			hashCode = context.hashCode();
		}
		return hashCode;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof Context)) {
			return false;
		}
		Context anoCtx = (Context) o;
		return context.equals(anoCtx.context);
	}
	
	@Override
	public String toString() {
		return "Context: " + context.toString();
	}

	private static final Context EMPTY_CONTEXT = new Context(Collections.emptyList());

	/**
	 * Return an empty context.
	 * 
	 */
	public static Context emptyContext() {
		return EMPTY_CONTEXT;
	}

	/**
	 * Compose a new context by a given context and a heap.
	 * The depth of new context is also required.
	 */
	public static Context newContext(Context c, Heap heap, int depth) {
		List<Heap> context;
		if (c.depth() < depth) {
			context = new ArrayList<>(c.depth() + 1);
			context.addAll(c.context);
		} else {
			context = new ArrayList<>(depth);
			for (int i = c.depth() - depth + 1; i < c.depth(); ++i) {
				context.add(c.context.get(i));
			}
		}
		context.add(heap);
		return new Context(context);
	}

}
