/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.oag.context;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import bean.oag.OAG;
import bean.pta.Heap;
import bean.util.Pair;

/**
 * Compute the contexts of each heap object via OAG.
 * 
 */
public abstract class ContextSelector {

	protected int depth; // the depth of context
	protected Map<Heap, Set<Context>> contextMap;
	protected Map<Pair<Context, Heap>, Set<Pair<Context, Heap>>> allocation =
			new HashMap<>();
	
	protected ContextSelector(OAG oag) {
		this(oag, Integer.MAX_VALUE);
	}

	protected ContextSelector(OAG oag, int depth) {
		this.depth = depth;
		selectContext(oag);
	}

	public int depth() {
		return depth;
	}

	public Set<Context> contextsOf(Heap heap) {
		return contextMap.get(heap);
	}
	
	public Set<Pair<Context, Heap>> allocatedBy(Context ctx, Heap heap) {
		return allocation.get(new Pair<>(ctx, heap));
	}

	protected abstract void selectContext(OAG oag);

	protected void addAllocation(Context ctx, Heap heap, Context newCtx, Heap succ) {
		Pair<Context, Heap> csheap = new Pair<>(ctx, heap);
		Set<Pair<Context, Heap>> heaps = allocation.get(csheap);
		if (heaps == null) {
			allocation.put(csheap, new HashSet<>());
		}
		allocation.get(csheap).add(new Pair<>(newCtx, succ));
	}
}
