/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.oag.context;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Set;

import bean.oag.OAG;
import bean.oag.Node;
import bean.pta.Heap;
import bean.util.Pair;

/**
 * Traditional k-obj analysis.
 *
 */
public class TraditionalContextSelector extends ContextSelector {

	public TraditionalContextSelector(OAG oag) {
		super(oag);
	}
	
	public TraditionalContextSelector(OAG oag, int depth) {
		super(oag, depth);
	}
	
	@Override
	protected void selectContext(OAG oag) {
		contextMap = new HashMap<>();
		Queue<Pair<Node, Context>> worklist = new LinkedList<>();
		Context emptyCtx = Context.emptyContext();
		// 1. initial work-list: <rootNode, emptyContext>
		oag.rootNodes().forEach(
				root -> worklist.add(new Pair<>(root,
						Context.newContext(emptyCtx, root.getHeap(), depth))));
		// 2. propagate contexts
		while (!worklist.isEmpty()) {
			Pair<Node, Context> pair = worklist.poll();
			Node node = pair.getFirst();
			Context ctx = pair.getSecond();
			Heap heap = node.getHeap();
			if (!contextMap.containsKey(heap)) {
				contextMap.put(heap, new HashSet<>());
			}
			if (contextMap.get(heap).add(ctx)) {
				oag.getSuccsOf(node).forEach(succ -> {
					Context newCtx = Context.newContext(ctx, heap, depth);
					Set<Context> ctxs = contextMap.get(succ);
					if (ctxs == null || !ctxs.contains(newCtx)) {
						addAllocation(ctx, heap, newCtx, succ.getHeap());
						worklist.add(new Pair<>(succ, newCtx));
					}
				});
			}
		}
	}

}
