/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.pta.doop;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * A class that represents the database of Doop.
 *
 */
class DataBase {

	private static final String CMD = "bloxbatch -db %s -query %s";
	private static final String SEP = ", ";
	
	/** The directory of Doop database */
	private final File dbDir;
	
	DataBase(File dbDir) {
		this.dbDir = dbDir;
	}
	
	/** Return the results of the give query. */
	List<List<String>> query(String query) throws IOException {
		String cmd = String.format(CMD, dbDir.getAbsolutePath(), query);
		Runtime rt = Runtime.getRuntime();
		Process proc = rt.exec(cmd);
		BufferedReader reader = new BufferedReader(
				new InputStreamReader(proc.getInputStream()));
		List<List<String>> result = new LinkedList<>();
		String line = null;
		while ((line = reader.readLine()) != null) {
			result.add(line2list(line));
		}
		try {
			proc.waitFor();
		} catch (InterruptedException e) {
			throw new RuntimeException("Exception during query: " + query);
		}
		reader.close();
		return result;
	}
	
	private List<String> line2list(String line) {
		return Arrays.asList(line.trim().split(SEP));
	}
	
}
