/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.pta.doop;

import bean.pta.Heap;
import bean.pta.Method;

/**
 * In Doop, a heap is represented by a String.
 */
class DoopHeap extends Heap {

	private final String heap;
	private final Method inMethod;
	
	DoopHeap(String heap, Method inMethod) {
		this.heap = heap;
		this.inMethod = inMethod;
	}
	
	@Override
	public int hashCode() {
		return heap.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof DoopHeap)) {
			return false;
		}
		DoopHeap anoHeap = (DoopHeap) obj;
		return heap.equals(anoHeap.heap);
	}
	
	@Override
	public String toString() {
		return heap;
	}

	@Override
	public Method inMethod() {
		return inMethod;
	}

}
