/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.pta.doop;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import bean.pta.Method;
import bean.pta.Variable;

/**
 * In Doop, a method is represented by its full signature: a string.
 */
class DoopMethod extends Method {

	private final String method;
	private final String name;
	private final Variable thisVar;
	private final boolean isStatic;
	
	DoopMethod(String methodSig, Variable thisVar, boolean isStatic) {
		this.method = methodSig;
		this.name = extractMethodName(methodSig);
		this.thisVar = thisVar;
		this.isStatic = isStatic;
	}
	
	@Override
	public int hashCode() {
		return method.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof DoopMethod)) {
			return false;
		}
		DoopMethod anoMeth = (DoopMethod) obj;
		return method.equals(anoMeth.method);
	}
	
	@Override
	public String toString() {
		return method;
	}

	@Override
	public Variable getThis() {
		return thisVar;
	}
	
	@Override
	public String getName() {
		return name;
	}
	
	@Override
	public boolean isStatic() {
		return isStatic;
	}
	

	private static final Pattern mtdNamePattern =
			Pattern.compile(".+\\s.+\\s(.+)\\(.*");
	
	private static String extractMethodName(String methSig) {
		Matcher matcher = mtdNamePattern.matcher(methSig);
		if (matcher.matches()) {
			return matcher.group(1);
		} else {
			throw new RuntimeException("extractMethodName error");
		}
	}
	
}
