/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.pta.doop;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import bean.pta.Heap;
import bean.pta.PointsToSet;
import bean.pta.Variable;

class DoopPointsToSet implements PointsToSet {

	// Query predicates.
	private static final String VARPOINTSTO = "Stats:Simple:InsensVarPointsTo";
	
	private Set<Variable> vars = new HashSet<>();
	private Set<Heap> heaps = new HashSet<>();
	private Map<Variable, Set<Heap>> p2set = new HashMap<>();
	
	DoopPointsToSet(DataBase db,
			VariableManager varManager, HeapManager heapManager) throws IOException {
		List<List<String>> varpt = db.query(VARPOINTSTO);
		varpt.forEach(list -> {
			String heapName = list.get(0);
			String varName = list.get(1);
			Variable var = varManager.getVariable(varName);
			Heap heap = heapManager.getHeap(heapName);
			vars.add(var);
			heaps.add(heap);
			addPointsTo(var, heap);
		});
	}

	@Override
	public Set<Variable> allVariables() {
		return vars;
	}

	@Override
	public Set<Heap> allHeaps() {
		return heaps;
	}

	@Override
	public Set<Heap> getPointsToSetOf(Variable var) {
		Set<Heap> heaps = p2set.get(var);
		return heaps == null ? Collections.emptySet() : heaps;
	}
	
	private void addPointsTo(Variable var, Heap heap) {
		if (!p2set.containsKey(var)) {
			p2set.put(var, new HashSet<>());
		}
		p2set.get(var).add(heap);
	}
	
}
