/* Bean - Making k-Object-Sensitive Pointer Analysis More Precise with Still k-Limiting
 *
 * Copyright (C) 2016 Tian Tan, Yue Li, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package bean.util;

import java.util.Objects;

public class Triple<T1, T2, T3> {

	private final T1 first;
	private final T2 second;
	private final T3 third;

	public Triple(T1 first, T2 second, T3 third) {
		this.first = first;
		this.second = second;
		this.third = third;
	}

	public T1 getFirst() {
		return first;
	}

	public T2 getSecond() {
		return second;
	}
	
	public T3 getThird() {
		return third;
	}

	@Override
	public int hashCode() {
		return Objects.hash(first, second, third);
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof Triple) {
			Triple<?, ?, ?> anoTriple = (Triple<?, ?, ?>) o;
			return Objects.equals(first, anoTriple.first)
					&& Objects.equals(second, anoTriple.second)
					&& Objects.equals(third, anoTriple.third);
		}
		return false;
	}
	
	@Override
	public String toString() {
		return "<"
				+ Objects.toString(first) + ", "
				+ Objects.toString(second) + ", "
				+ Objects.toString(third)
				+ ">";
	}
	
}
